package jp.cox.swf.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;

/**
 * SWF\IuWFNgB
 * 
 * @author Shunichi Todoroki
 */
public class Swf {
	private static final int FIX = 3;
	private static final int VERSION = 1;
	private static final int FILE_SIZE = 4;
	private int RECT;
	private static final int FPS = 2;
	private static final int FRAMES = 2;
	private int FIRST_TAG = 6;
	private int HEAD_SIZE;
	private byte[] head;
	private byte[] body;

	public Swf(byte[] b) {
		calcHeadSize(b);
		head = new byte[HEAD_SIZE];
		System.arraycopy(b, 0, head, 0, HEAD_SIZE);
		body = new byte[b.length - HEAD_SIZE];
		System.arraycopy(b, HEAD_SIZE, body, 0, body.length);
	}

	private void calcHeadSize(byte[] b) {
		RECT = rectSize(b);
		FIRST_TAG = firstTagSize(b);
		HEAD_SIZE = FIX + VERSION + FILE_SIZE + RECT + FPS + FRAMES
				+ FIRST_TAG;
	}

	private int rectSize(byte[] b) {
		int i = b[FIX + VERSION + FILE_SIZE] >> 3;
		int n = (int) ((((8 - ((i * 4 + 5) & 7)) & 7) + i * 4 + 5) / 8);
		return n;
	}
	
	private int firstTagSize(byte[] b) {
		int index = FIX + VERSION + FILE_SIZE + RECT + FPS + FRAMES;
		byte[] firstTagHead = new byte[2];
		firstTagHead[0] = b[index++];
		firstTagHead[1] = b[index];
		int i = from16LittleEndianByte(firstTagHead) & 0x3f;
		return i + 2;
	}

	/**
	 * SWF̃oCg\Ԃ܂B
	 * 
	 * @return SWF̃oCg
	 */
	public byte[] getBinary() {
		byte[] b = new byte[head.length + body.length];
		System.arraycopy(head, 0, b, 0, head.length);
		System.arraycopy(body, 0, b, head.length, body.length);
		return b;
	}

	/**
	 * SWFInputXg[ŕԂ܂B
	 * 
	 * @return SWFInputXg[
	 */
	public InputStream getInputStream() {
		byte[] b = new byte[head.length + body.length];
		System.arraycopy(head, 0, b, 0, head.length);
		System.arraycopy(body, 0, b, head.length, body.length);
		return new ByteArrayInputStream(b);
	}

	/**
	 * SWFɃp[^ƃp[^l𖄂ߍ݂܂B
	 * 
	 * @param name p[^[
	 * @param value p[^[l
	 */
	public void addParameter(String name, String value) {
		if(name == null || value == null) {
			throw new NullPointerException();
		}
		byte[] paramTag = createParameterTag(name, value);
		//t@CTCYς
		byte[] sizeByte = new byte[4];
		System.arraycopy(head, (FIX + VERSION), sizeByte, 0, 4);
		int fileSize = from32LittleEndianByte(sizeByte);
		fileSize += paramTag.length;
		System.arraycopy(to32LittleEndianByte(fileSize), 0, head, (FIX + VERSION), 4);
		//bodyparamTagǉ
		byte[] newBody = new byte[body.length + paramTag.length];
		System.arraycopy(paramTag, 0, newBody, 0, paramTag.length);
		System.arraycopy(body, 0, newBody, paramTag.length, body.length);
		body = newBody;
	}

	/*
	 * p[^̖OƒlɎAߍރ^O𐶐ĕԂ܂B
	 * 
	 * @param name p[^[
	 * @param value p[^[l
	 * @return p[^[^ÕoCgz
	 */
	private byte[] createParameterTag(String name, String value) {
		byte[] nameBytes = name.getBytes();
		byte[] valueBytes = value.getBytes();
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		bout.write(0x3f);
		bout.write(0x03);
		bout.write(to32LittleEndianByte(nameBytes.length + valueBytes.length + 12),0,4);
		//name
		bout.write(0x96);
		bout.write(to16LittleEndianByte(name.length()+2),0,2);
		bout.write(0x00);
		bout.write(nameBytes,0,nameBytes.length);
		bout.write(0x00);
		//value
		bout.write(0x96);
		bout.write(to16LittleEndianByte(value.length()+2),0,2);
		bout.write(0x00);
		bout.write(valueBytes,0,valueBytes.length);
		bout.write(0x00);
		//last
		bout.write(0x1d);
		bout.write(0x00);
		return bout.toByteArray();
	}
	
	private byte[] to16LittleEndianByte(int target) {
		byte[] b = new byte[2];
		b[0] = (byte)target;
		b[1] = (byte)(target >> 8);
		return b;
	}
	
	private byte[] to32LittleEndianByte(int target) {
		byte[] b = new byte[4];
		b[0] = (byte)target;
		b[1] = (byte)(target >> 8);
		b[2] = (byte)(target >> 16);
		b[3] = (byte)(target >> 24);
		return b;
	}
	
	private int from16LittleEndianByte(byte[] target) {
		int r = 0;
		r += (((int) target[1]) & 0xff);
		r <<= 8;
		r += (((int) target[0]) & 0xff);
		return r;
	}
	
	private int from32LittleEndianByte(byte[] target) {
		int r = 0;
		r += (((int) target[3]) & 0xff);
		r <<= 8;
		r += (((int) target[2]) & 0xff);
		r <<= 8;
		r += (((int) target[1]) & 0xff);
		r <<= 8;
		r += (((int) target[0]) & 0xff);
		return r;
	}
}
